/**
 * Copyright (c) 2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.registry_overlay.api;

import java.io.IOException;
import java.io.Serializable;

import com.hazelcast.nio.ObjectDataInput;
import com.hazelcast.nio.ObjectDataOutput;
import com.hazelcast.nio.serialization.IdentifiedDataSerializable;
import com.hazelcast.nio.serialization.Portable;

/**
 * This classes is used to store data into the registry without depending on too much petals specific classes.
 * 
 * It also enables us to take advantage of {@link IdentifiedDataSerializable} implementation which improves performance
 * of serialization/deserializaton compared to basic Java {@link Serializable}.
 * 
 * TODO Move to {@link Portable} when 3.6.3 is released (because for now it does not support nested queries): it will
 * also improve performances regarding queries!
 * 
 * @author vnoel
 *
 */
public class StoredRegistryEndpoint implements IdentifiedDataSerializable {

    public enum Type {
        INTERNAL, EXTERNAL
    }

    private Type type;

    private String endpointName;

    private String serviceName;

    private String[] interfaces;

    private String container;

    private String component;

    StoredRegistryEndpoint() {
        // used by RegistryEndpointFactory
    }

    public StoredRegistryEndpoint(final Type type, final String endpointName, final String serviceName,
            final String[] interfaces, final String container, final String component) {
        this.type = type;
        this.endpointName = endpointName;
        this.serviceName = serviceName;
        this.interfaces = interfaces;
        this.container = container;
        this.component = component;
    }

    @Override
    public int getFactoryId() {
        return RegistryEndpointFactory.REGISTRY_ENDPOINT_FACTORY_ID;
    }

    @Override
    public int getId() {
        return RegistryEndpointFactory.STORED_REGISTRY_ENDPOINT_CLASS_ID;
    }

    @Override
    public void writeData(final ObjectDataOutput out) throws IOException {
        out.writeObject(type);
        out.writeUTF(endpointName);
        out.writeUTF(serviceName);
        out.writeUTFArray(interfaces);
        out.writeUTF(container);
        out.writeUTF(component);
    }

    @Override
    public void readData(final ObjectDataInput in) throws IOException {
        type = in.readObject();
        endpointName = in.readUTF();
        serviceName = in.readUTF();
        interfaces = in.readUTFArray();
        container = in.readUTF();
        component = in.readUTF();
    }

    public Type getType() {
        return type;
    }

    public String getEndpointName() {
        return endpointName;
    }

    public String getServiceName() {
        return serviceName;
    }

    public String[] getInterfaces() {
        return interfaces;
    }

    public String getContainer() {
        return container;
    }

    public String getComponent() {
        return component;
    }
}
